#!/usr/local/bin/perl -w
#
# $Id: coordina.pl,v 1.5 2000/11/09 19:44:52 jens Exp $
#
# Copyright (C) 2001 Jens Thoms Toerring
#
# g_rhf is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# g_rhf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with g_rhf; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Parses a MOPAC or AMPAC RHF or Hyperchem/Plato ZINDO/S result file and
# prints the type of the atom and its three cartesian coordinates (from the
# second, possibly optimized set of coordinates for AMPAC/MOPAC files).


use strict;

local *F;
my $method;
my $in_coordinates = 0;
my @list;


open( F, "< $ARGV[ 0 ]" ) or exit;

# Figure out the type of the input file

while ( <F> ) {

	if ( /\b((MOPAC)|(AMPAC)|(HyperChem))\b/ ) {
		$method = uc $1;
		last;
	}
}

exit unless defined $method;

if ( $method =~ /(AMPAC)|(MOPAC)/ ) {

	my $is_second_set = 0;

	# Goto second section with cartesian coordinates

	while ( <F> ) {
		next unless /CARTESIAN COORDINATES/i;
		unless ( $is_second_set ) {
			$is_second_set = 1;
			next;
		}

		# Read in all cartesian coordinates and print them

		while ( <F> ) {
			next unless /^\s*\d+\s+([A-Z]{1,2}|[+-])\s+/i or $in_coordinates;
			next if /^\s*\d+\s+[+-]\s+/;
			last unless /^\s*\d+\s+([A-Z]{1,2}|[+-])\s+/i;
			$in_coordinates = 1;
			s/\s+/ /g;
			/\s*(\d+\s+.*)\b\s*/;
			print "$1\n";
		}

		last;
	}
} else {

	my @atom_type;

	# ZINDO files don't have atom type names (e.g. H, O or Cl) in the list of
	# coordinates but atomic charges - but the LCAO list has also the names,
	# so we get it from there

	while ( <F> ) {
		if ( /^\s*((S)|(PX)|(PY)|(PZ))\s+[A-Z]{1,2}\s+[0-9]+/i ) {
			/\s*([SP].*\d)\b\s*/;       # get rid of leading or trailing spaces
			@list = split /\s+/, $1;
			$atom_type[ $list[ 2 ] ] = ucfirst $list[ 1 ];
		} else {
		   last if @atom_type;
		}
	}

	# Wait for title line of list of coordinates

	while ( <F> ) {
		last if /NET CHARGES AND COORDINATES/
	}

	# Read in the coordinates and print them

	while ( <F> ) {
		next unless /\s*\d+\s+\d+\s+/ or $in_coordinates;
		last unless /\s*\d+\s+\d+\s+/;
		$in_coordinates = 1;
		s/s+/ /g;
		/\s*(.*)\b\s*/;
		@list = split /\s+/, $1;
		print "$list[ 0 ] $atom_type[ $list[ 0 ] ] " .
			  "$list[ 3 ] $list[ 4 ] $list[ 5 ]\n";
	}
}

close F;
