/*********************************************************************

 $Id: diamag.c,v 1.6 2000/11/03 19:03:26 jens Exp $

  Copyright (C) 2001 Jens Thoms Toerring

  g_rhf is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  g_rhf is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with g_rhf; see the file COPYING.  If not, write to
  the Free Software Foundation, 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.

 This file contains routines for the diamagnetic part of the g tensor

***********************************************************************/


#include "g_rhf.h"


/*******************************************/
/* Calculates the diamagnetic contribution */
/* to the G tensor using Angstl's method.  */
/*******************************************/


void calculate_diamagnetic_part_Angstl( DMATRIX g2, ATOM *atom, NUMS *n )
{
	int l, m, K;
	double temp;


    /* We start with the diagonal elements ... */

    for ( l = X; l <= Z; l++ )      /* ... looping over x, y and z direction */
    {

        for ( temp = 0.0, K = 0; K < n->atoms; K++ )
		{
            temp += 2.0 / 3.0 * atom[ K ].sigma_s
				    * sqr( atom[ K ].c[ S ][ n->SOMO ] );

			if ( atom[ K ].n == 1 )
				continue;

			temp += atom[ K ].sigma_p
			      * (   0.4 * sqr(atom[K].c[l + PX][n->SOMO])
				      + 0.8 * (  sqr(atom[K].c[((l + 1) % 3) + PX][n->SOMO])
					           + sqr(atom[K].c[((l + 2) % 3) + PX][n->SOMO])));
		}

		g2[ l ][ l ] += 0.25 * temp;


    /* Now we proceed with the offdiagonal elements ... */

		for ( m = l + 1; m <= Z; m++ )
		{
			for ( temp = 0.0, K = 0; K < n->atoms; K++ )
			{
				if ( atom[ K ].n == 1 )
					continue;

                temp += 0.4 * atom[ K ].zeta_p
					    * atom[ K ].c[ l + PX ][ n->SOMO ]
					    * atom[ K ].c[ m + PX ][ n->SOMO ];
			}

			g2[ m ][ l ] = g2[ l ][ m ] -= 0.25 * temp;

		} /* End of loop over all m = x, y, z */
	} /* End of loop over all l = x, y, z */
}


/***********************************************************/
/* Calculates the diamagnetic contribution to the G tensor */
/***********************************************************/

void calculate_diamagnetic_part( DMATRIX g2, ATOM *atom, NUMS *n )
{
	int l, m, K;
	double temp;


    /* We start with the diagonal elements ... */

    for ( l = X; l <= Z; l++ )      /* ... looping over x, y and z direction */
    {
        for ( temp = 0.0, K = 0; K < n->atoms; K++ )
		{
            temp += ( ( atom[ K ].n == 1 ) ? 1.0 : ( 5.0 / 3.0 ) )
				    * atom[ K ].xi
			        * sqr( atom[ K ].c[ S ][ n->SOMO ] ) / atom[ K ].zeta_s;

			if ( atom[ K ].n == 1 )
				continue;

			temp += atom[ K ].xi / atom[ K ].zeta_p
			      * (   sqr( atom[ K ].c[ l + PX ][ n->SOMO ] )
				      + 2.0 * (  sqr(atom[K].c[((l + 1) % 3) + PX][n->SOMO])
					           + sqr(atom[K].c[((l + 2) % 3) + PX][n->SOMO])));
		}

		g2[ l ][ l ] += 0.25 * temp;

		/* Now we proceed with the offdiagonal elements ... */

		for ( m = l + 1; m <= Z; m++ )
		{
			for ( temp = 0.0, K = 0; K < n->atoms; K++ )
			{
				if (atom[ K ].n == 1 )
					continue;

                temp += atom[K].xi / atom[K].zeta_p
					 * atom[K].c[l + PX][n->SOMO] * atom[K].c[m + PX][n->SOMO];
			}

			g2[ m ][ l ] = g2[ l ][ m ] -= 0.25 * temp;

		} /* End of loop over all m = x, y, z */
	} /* End of loop over all l = x, y, z */
}
