/***********************************************************************

  $Id: e_util.c,v 1.3 2000/11/03 19:06:04 jens Exp $

  Copyright (C) 2001 Jens Thoms Toerring

  g_rhf is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  g_rhf is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with g_rhf; see the file COPYING.  If not, write to
  the Free Software Foundation, 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.

  This file contains routines used in the calculation of the
  energy corrections.

************************************************************************/


#include "g_rhf.h"



/* Definitions needed for this file */

#define q_q       0                 /* different types of multipole- */
#define q_uz      1                 /* multipole interactions */
#define uz_q      2
#define q_Qpp     3
#define Qpp_q     4
#define q_Qzz     5
#define Qzz_q     6
#define up_up     7
#define uz_uz     8
#define up_Qpz    9
#define Qpz_up   10
#define uz_Qpp   11
#define Qpp_uz   12
#define uz_Qzz   13
#define Qzz_uz   14
#define Qpp_Qpp  15
#define Qxx_Qyy  16
#define Qpp_Qzz  17
#define Qzz_Qpp  18
#define Qzz_Qzz  19
#define Qpz_Qpz  20
#define Qxy_Qxy  21



/****************************************************************************/
/* Calculates the one-center two-electron interactions. One should assume   */
/* that they are identical to the two-center two-electron interactions with */
/* the interatomic distance set to zero, as also can be concluded from the  */
/* paper M. J. S. Dewar and W. Thiel, Theoret. Chim. Acta (Berl.) 46, 89,   */
/* (1977). Unfortunately this is not the way it is handled in all the MO    */
/* programs. So in order not to get deviating results we do it the way all  */
/* other people do it ...                                                   */
/****************************************************************************/

void make_oc_ints(double tc_int[4][4][4][4], ATOM *atom)
{
    /* Calculate one-center integral in case of all 4 states being S states */

    tc_int[S][S][S][S] = atom->Gss;                    /* (  S  S |  S  S ) */

    /* If atom has n = 1 we are finished */

    if (atom->n == 1)
        return;

    /* Now calculate one-center integrals for the case that n > 1 */

	tc_int[S][S][PX][PX] = tc_int[S][S][PY][PY] =      /* (  S  S |  P  P ) */
	tc_int[S][S][PZ][PZ] = tc_int[PX][PX][S][S] =
	tc_int[PY][PY][S][S] = tc_int[PZ][PZ][S][S] = atom->Gsp;

	tc_int[S][S][S][PZ]  = tc_int[S][S][PZ][S]  =      /* (  S  S |  S PZ ) */
	tc_int[S][PZ][S][S]  = tc_int[PZ][S][S][S]  = 0.0;

    tc_int[S][PX][PX][PZ] = tc_int[S][PY][PY][PZ] =    /* (  S Pp | Pp PZ ) */
    tc_int[PX][S][PX][PZ] = tc_int[PY][S][PY][PZ] =
    tc_int[S][PX][PZ][PX] = tc_int[S][PY][PZ][PY] =
    tc_int[PX][S][PZ][PX] = tc_int[PY][S][PZ][PY] =
    tc_int[PX][PZ][S][PX] = tc_int[PY][PZ][S][PY] =
    tc_int[PZ][PX][S][PX] = tc_int[PZ][PY][S][PY] =
    tc_int[PX][PZ][PX][S] = tc_int[PY][PZ][PY][S] =
    tc_int[PZ][PX][PX][S] = tc_int[PZ][PY][PY][S] = 0.0;

    tc_int[S][PZ][PX][PX] = tc_int[S][PZ][PY][PY] =    /* (  S PZ | Pp Pp ) */
    tc_int[PZ][S][PX][PX] = tc_int[PZ][S][PY][PY] =
    tc_int[PX][PX][S][PZ] = tc_int[PY][PY][S][PZ] =
    tc_int[PX][PX][PZ][S] = tc_int[PY][PY][PZ][S] = 0.0;

    tc_int[S][PZ][PZ][PZ] = tc_int[PZ][S][PZ][PZ] =    /* (  S PZ | PZ PZ ) */
    tc_int[PZ][PZ][S][PZ] = tc_int[PZ][PZ][PZ][S] = 0.0;

    tc_int[S][PX][S][PX] = tc_int[S][PY][S][PY] =      /* (  S  P |  S  P ) */
    tc_int[PX][S][S][PX] = tc_int[PY][S][S][PY] =
    tc_int[S][PX][PX][S] = tc_int[S][PY][PY][S] =
    tc_int[PX][S][PX][S] = tc_int[PY][S][PY][S] =
    tc_int[S][PZ][S][PZ] = tc_int[PZ][S][S][PZ] =
    tc_int[S][PZ][PZ][S] = tc_int[PZ][S][PZ][S] = atom->Hsp;

    tc_int[PX][PX][PX][PX] = tc_int[PY][PY][PY][PY] =  /* (  P  P |  P  P ) */
	tc_int[PZ][PZ][PZ][PZ] = atom->Gpp;

    tc_int[PX][PX][PY][PY] = tc_int[PY][PY][PX][PX] =  /* (  P  P | P' P' ) */
    tc_int[PX][PX][PZ][PZ] = tc_int[PY][PY][PZ][PZ] =
    tc_int[PZ][PZ][PX][PX] = tc_int[PZ][PZ][PY][PY] = atom->Gp2;

	/* I think that the following formulae are definitely wrong
	   but since everybody is using them ... */

    tc_int[PX][PZ][PX][PZ] = tc_int[PY][PZ][PY][PZ] =  /*  (  P P' |  P P' ) */
    tc_int[PZ][PX][PX][PZ] = tc_int[PZ][PY][PY][PZ] =
    tc_int[PX][PZ][PZ][PX] = tc_int[PY][PZ][PZ][PY] =
    tc_int[PZ][PX][PZ][PX] = tc_int[PZ][PY][PZ][PY] =
    tc_int[PX][PY][PX][PY] = tc_int[PY][PX][PX][PY] =
    tc_int[PX][PY][PY][PX] = tc_int[PY][PX][PY][PX] =
		                                       0.5 * (atom->Gpp - atom->Gp2);
}


/***************************************************************************/
/* Calculates the 22 multipole-multipole integrals according to formulae   */
/* in M. J. S. Dewar, W. Thiel, Theoret. Chim. Acta (Berl.) 46, 89 (1977)  */
/* with corrected formulae for [uz,uz] and [Qpz,Qpz].                      */
/* For [Qxy,Qxy] we use a wrong formula but which is used in all MO pro-   */
/* grams instead of the correct one from Dewar and Thiel's paper.          */
/* Input parameter : pointer to array of the multipole-multipole integrals */
/*                   pointer to the array of structures of all atoms       */
/*                   interatomic distance                                  */
/*                   number of first (K) and second (L) atom               */
/***************************************************************************/

void make_integrals_1(double *m_int, ATOM *atom, double R, int K, int L)
{
    double r0A, r1A, r2A, D1A, D2A,        /* temporary variables */
           r0B, r1B, r2B, D1B, D2B,
           R2, a01, a02, a10, a11, a12, a20, a21, a22;


    r0A = atom[K].rho_0;      /* temporary variables are used to speed up */
    r1A = atom[K].rho_1;      /* access a bit and to ease readability of  */
    r2A = atom[K].rho_2;      /* this rather boring piece of program ...  */
    D1A = atom[K].D1;
    D2A = atom[K].D2;

    r0B = atom[L].rho_0;
    r1B = atom[L].rho_1;
    r2B = atom[L].rho_2;
    D1B = atom[L].D1;
    D2B = atom[L].D2;

    R2 = sqr( R );            /* square of interatomic distance */

    /* Calculate interaction between two monopoles */

    m_int[q_q] = 1.0 / sqrt( R2 + sqr( r0A + r0B ));

    /* If both atoms have n = 1 we're already finished */

    if ((atom[K].n == 1) && (atom[L].n == 1))
        return;

    /* Calculate interactions of a monopole with a dipole or a quadru-
       pole for the case that at least one of the atoms has n > 1 */

    if (atom[L].n > 1)
    {
        a01 = sqr( r0A + r1B );
        a02 = sqr( r0A + r2B );

        m_int[q_uz] = 0.5 *
            (   1.0 / sqrt( sqr( R + D1B ) + a01 )
              - 1.0 / sqrt( sqr( R - D1B ) + a01 ));

        m_int[q_Qpp] = 0.5 *
            (   1.0 / sqrt( R2 + sqr( 2.0 * D2B ) + a02 )
              - 1.0 / sqrt( R2                    + a02 ));

        m_int[q_Qzz] = 0.25 *
            (   1.0 / sqrt( sqr( R + 2.0 * D2B ) + a02 )
              + 1.0 / sqrt( sqr( R - 2.0 * D2B ) + a02 )
              - 2.0 / sqrt( R2                   + a02 ));
    }

    if (atom[K].n > 1)
    {
        a10 = sqr( r1A + r0B );
        a20 = sqr( r2A + r0B );

        m_int[uz_q] = - 0.5 *
            (   1.0 / sqrt( sqr( R + D1A ) + a10 )
              - 1.0 / sqrt( sqr( R - D1A ) + a10 ));

        m_int[Qpp_q] = 0.5 *
            (   1.0 / sqrt( R2 + sqr( 2.0 * D2A ) + a20 )
              - 1.0 / sqrt( R2                    + a20 ));

        m_int[Qzz_q] = 0.25 *
            (   1.0 / sqrt( sqr( R + 2.0 * D2A ) + a20 )
              + 1.0 / sqrt( sqr( R - 2.0 * D2A ) + a20 )
              - 2.0 / sqrt( R2                   + a20 ));
    }

    /* If one of the atoms has n = 1 we're finished */

    if ((atom[K].n == 1) || (atom[L].n == 1))
        return;

    /* Otherwise calculate all remaining integrals */

    a11 = sqr( r1A + r1B );
    a12 = sqr( r1A + r2B );
    a21 = sqr( r2A + r1B );
    a22 = sqr( r2A + r2B );

    m_int[up_up] = 0.5 *
        (   1.0 / sqrt( R2 + sqr( D1A - D1B ) + a11 )
          - 1.0 / sqrt( R2 + sqr( D1A + D1B ) + a11 ));

    m_int[uz_uz] = 0.25 *
        (   1.0 / sqrt( sqr( R + D1A - D1B ) + a11 )
          - 1.0 / sqrt( sqr( R + D1A + D1B ) + a11 )
          - 1.0 / sqrt( sqr( R - D1A - D1B ) + a11 )
          + 1.0 / sqrt( sqr( R - D1A + D1B ) + a11 ));

    m_int[up_Qpz] = 0.25 *
        ( - 1.0 / sqrt( sqr( R - D2B ) + sqr( D1A - D2B ) + a12 )
          + 1.0 / sqrt( sqr( R - D2B ) + sqr( D1A + D2B ) + a12 )
          + 1.0 / sqrt( sqr( R + D2B ) + sqr( D1A - D2B ) + a12 )
          - 1.0 / sqrt( sqr( R + D2B ) + sqr( D1A + D2B ) + a12 ));

    m_int[Qpz_up] = - 0.25 *
        ( - 1.0 / sqrt( sqr( R - D2A ) + sqr( D1B - D2A ) + a21 )
          + 1.0 / sqrt( sqr( R - D2A ) + sqr( D1B + D2A ) + a21 )
          + 1.0 / sqrt( sqr( R + D2A ) + sqr( D1B - D2A ) + a21 )
          - 1.0 / sqrt( sqr( R + D2A ) + sqr( D1B + D2A ) + a21 ));

    m_int[uz_Qpp] = 0.25 *
        ( - 1.0 / sqrt( sqr( R + D1A ) + sqr( 2.0 * D2B ) + a12 )
          + 1.0 / sqrt( sqr( R - D1A ) + sqr( 2.0 * D2B ) + a12 )
          + 1.0 / sqrt( sqr( R + D1A )                    + a12 )
          - 1.0 / sqrt( sqr( R - D1A )                    + a12 ));

    m_int[Qpp_uz] = - 0.25 *
        ( - 1.0 / sqrt( sqr( R + D1B ) + sqr( 2.0 * D2A ) + a21 )
          + 1.0 / sqrt( sqr( R - D1B ) + sqr( 2.0 * D2A ) + a21 )
          + 1.0 / sqrt( sqr( R + D1B )                    + a21 )
          - 1.0 / sqrt( sqr( R - D1B )                    + a21 ));

    m_int[uz_Qzz] = 0.125 *
        ( - 1.0 / sqrt( sqr( R + D1A - 2.0 * D2B ) + a12 )
          + 1.0 / sqrt( sqr( R - D1A - 2.0 * D2B ) + a12 )
          - 1.0 / sqrt( sqr( R + D1A + 2.0 * D2B ) + a12 )
          + 1.0 / sqrt( sqr( R - D1A + 2.0 * D2B ) + a12 )
          + 2.0 / sqrt( sqr( R + D1A )             + a12 )
          - 2.0 / sqrt( sqr( R - D1A )             + a12 ));

    m_int[Qzz_uz] = - 0.125 *
        ( - 1.0 / sqrt( sqr( R + D1B - 2.0 * D2A ) + a21 )
          + 1.0 / sqrt( sqr( R - D1B - 2.0 * D2A ) + a21 )
          - 1.0 / sqrt( sqr( R + D1B + 2.0 * D2A ) + a21 )
          + 1.0 / sqrt( sqr( R - D1B + 2.0 * D2A ) + a21 )
          + 2.0 / sqrt( sqr( R + D1B )             + a21 )
          - 2.0 / sqrt( sqr( R - D1B )             + a21 ));

    m_int[Qpp_Qpp] = 0.125 *
        (   1.0 / sqrt( R2 + 4.0 * sqr( D2A - D2B ) + a22 )
          + 1.0 / sqrt( R2 + 4.0 * sqr( D2A + D2B ) + a22 )
          - 2.0 / sqrt( R2 + sqr( 2.0 * D2A )       + a22 )
          - 2.0 / sqrt( R2 + sqr( 2.0 * D2B )       + a22 )
          + 2.0 / sqrt( R2                          + a22 ));

    m_int[Qxx_Qyy] = 0.25 *
        (   1.0 / sqrt( R2 + sqr( 2.0 * D2A ) + sqr( 2.0 * D2B ) + a22 )
          - 1.0 / sqrt( R2 + sqr( 2.0 * D2A )                    + a22 )
          - 1.0 / sqrt( R2 + sqr( 2.0 * D2B )                    + a22 )
          + 1.0 / sqrt( R2                                       + a22 ));

    m_int[Qpp_Qzz] = 0.125 *
        (   1.0 / sqrt( sqr( R - 2.0 * D2B ) + sqr( 2.0 * D2A ) + a22 )
          + 1.0 / sqrt( sqr( R + 2.0 * D2B ) + sqr( 2.0 * D2A ) + a22 )
          - 1.0 / sqrt( sqr( R - 2.0 * D2B )                    + a22 )
          - 1.0 / sqrt( sqr( R + 2.0 * D2B )                    + a22 )
          - 2.0 / sqrt( R2 + sqr( 2.0 * D2A )                   + a22 )
          + 2.0 / sqrt( R2                                      + a22 ));

    m_int[Qzz_Qpp] = 0.125 *
        (   1.0 / sqrt( sqr( R - 2.0 * D2A ) + sqr( 2.0 * D2B ) + a22 )
          + 1.0 / sqrt( sqr( R + 2.0 * D2A ) + sqr( 2.0 * D2B ) + a22 )
          - 1.0 / sqrt( sqr( R - 2.0 * D2A )                    + a22 )
          - 1.0 / sqrt( sqr( R + 2.0 * D2A )                    + a22 )
          - 2.0 / sqrt( R2 + sqr( 2.0 * D2B )                   + a22 )
          + 2.0 / sqrt( R2                                      + a22 ));

    m_int[Qzz_Qzz] = 0.0625 *
        (   1.0 / sqrt( sqr( R + 2.0 * ( D2A - D2B )) + a22 )
          + 1.0 / sqrt( sqr( R + 2.0 * ( D2A + D2B )) + a22 )
          + 1.0 / sqrt( sqr( R - 2.0 * ( D2A + D2B )) + a22 )
          + 1.0 / sqrt( sqr( R - 2.0 * ( D2A - D2B )) + a22 )
          - 2.0 / sqrt( sqr( R + 2.0 * D2A )          + a22 )
          - 2.0 / sqrt( sqr( R - 2.0 * D2A )          + a22 )
          - 2.0 / sqrt( sqr( R + 2.0 * D2B )          + a22 )
          - 2.0 / sqrt( sqr( R - 2.0 * D2B )          + a22 )
          + 4.0 / sqrt( R2                            + a22 ));

    m_int[Qpz_Qpz] = 0.125 *
        (   1.0 / sqrt( sqr( R + D2A - D2B ) + sqr( D2A - D2B ) + a22 )
          - 1.0 / sqrt( sqr( R + D2A - D2B ) + sqr( D2A + D2B ) + a22 )
          - 1.0 / sqrt( sqr( R + D2A + D2B ) + sqr( D2A - D2B ) + a22 )
          + 1.0 / sqrt( sqr( R + D2A + D2B ) + sqr( D2A + D2B ) + a22 )
          - 1.0 / sqrt( sqr( R - D2A - D2B ) + sqr( D2A - D2B ) + a22 )
          + 1.0 / sqrt( sqr( R - D2A - D2B ) + sqr( D2A + D2B ) + a22 )
          + 1.0 / sqrt( sqr( R - D2A + D2B ) + sqr( D2A - D2B ) + a22 )
          - 1.0 / sqrt( sqr( R - D2A + D2B ) + sqr( D2A + D2B ) + a22 ));

	/* The following formula would be the correct one according to the paper
	   by Dewar and Thiel:

       m_int[Qxy_Qxy] = 0.25 *
                (   1.0 / sqrt( R2 + 2.0 * sqr( D2A - D2B )        + a22 )
                  + 1.0 / sqrt( R2 + 2.0 * sqr( D2A + D2B )        + a22 )
                  - 2.0 / sqrt( R2 + 2.0 * ( sqr(D2A) + sqr(D2B) ) + a22 ));

       but in order to get the same result for the integrals as AMPAC and
	   MOPAC etc. we have to use an obviously wrong one instead ... */

    m_int[Qxy_Qxy] = 0.5 * ( m_int[Qpp_Qpp] - m_int[Qxx_Qyy] );
}


/****************************************************************************/
/* Calculates the two-center integrals according to the formulae from       */
/* M. J. S. Dewar, W. Thiel, Theoret. Chim. Acta (Berl.) 46, 89 (1977)      */
/* Input parameter : pointer to array of 22 multipole-multipole integrals   */
/*                   pointer to the 4-dimensional array of the 256 possible */
/*                   two-center integrals                                   */
/*                   main quantum nunber of first atom A                    */
/*                   main quantum number of second atom B                   */
/****************************************************************************/

void make_integrals_2(double *m_int, double tc_int[4][4][4][4], int nA, int nB)
{
    /* Calculate two-center integrals in case of all 4 states being S states */

    tc_int[S][S][S][S] = m_int[q_q];

    /* If Atom A and B both have n = 1 we are finished */

    if ((nA == 1) && (nB == 1))
        return;

    /* Now calculate two-center-integrals for the case that at least
       one of the two atoms has a n > 1 */

    if (nA > 1)
    {
        tc_int[PX][PX][S][S] = tc_int[PY][PY][S][S] =
                               m_int[q_q] + m_int[Qpp_q];
        tc_int[PZ][PZ][S][S] = m_int[q_q] + m_int[Qzz_q];
        tc_int[S][PZ][S][S]  = tc_int[PZ][S][S][S] = m_int[uz_q];
    }

    if (nB > 1)
    {
        tc_int[S][S][PX][PX] = tc_int[S][S][PY][PY] =
                               m_int[q_q] + m_int[q_Qpp];
        tc_int[S][S][PZ][PZ] = m_int[q_q] + m_int[q_Qzz];
        tc_int[S][S][S][PZ]  = tc_int[S][S][PZ][S] = m_int[q_uz];
    }

    /* If one of the two atoms has n = 1 we're finished */

    if ((nA == 1) || (nB == 1))
        return;

    /* Finally calculate all the remaining two-center integrals */

    tc_int[S][PX][S][PX] = tc_int[S][PY][S][PY] =
    tc_int[PX][S][S][PX] = tc_int[PY][S][S][PY] =
    tc_int[S][PX][PX][S] = tc_int[S][PY][PY][S] =
    tc_int[PX][S][PX][S] = tc_int[PY][S][PY][S] = m_int[up_up];

    tc_int[S][PZ][S][PZ] = tc_int[PZ][S][S][PZ] =
    tc_int[S][PZ][PZ][S] = tc_int[PZ][S][PZ][S] = m_int[uz_uz];

    tc_int[S][PX][PX][PZ] = tc_int[S][PY][PY][PZ] =
    tc_int[PX][S][PX][PZ] = tc_int[PY][S][PY][PZ] =
    tc_int[S][PX][PZ][PX] = tc_int[S][PY][PZ][PY] =
    tc_int[PX][S][PZ][PX] = tc_int[PY][S][PZ][PY] = m_int[up_Qpz];

    tc_int[PX][PZ][S][PX] = tc_int[PY][PZ][S][PY] =
    tc_int[PZ][PX][S][PX] = tc_int[PZ][PY][S][PY] =
    tc_int[PX][PZ][PX][S] = tc_int[PY][PZ][PY][S] =
    tc_int[PZ][PX][PX][S] = tc_int[PZ][PY][PY][S] = m_int[Qpz_up];

    tc_int[S][PZ][PX][PX] = tc_int[S][PZ][PY][PY] =
    tc_int[PZ][S][PX][PX] = tc_int[PZ][S][PY][PY] =
                                      m_int[uz_q] + m_int[uz_Qpp];

    tc_int[PX][PX][S][PZ] = tc_int[PY][PY][S][PZ] =
    tc_int[PX][PX][PZ][S] = tc_int[PY][PY][PZ][S] =
                                      m_int[q_uz] + m_int[Qpp_uz];

    tc_int[S][PZ][PZ][PZ] = tc_int[PZ][S][PZ][PZ] =
                                      m_int[uz_q] + m_int[uz_Qzz];
    tc_int[PZ][PZ][S][PZ] = tc_int[PZ][PZ][PZ][S] =
                                      m_int[q_uz] + m_int[Qzz_uz];

    tc_int[PX][PX][PX][PX] = tc_int[PY][PY][PY][PY] =
        m_int[q_q] + m_int[q_Qpp] + m_int[Qpp_q] + m_int[Qpp_Qpp];

    tc_int[PX][PX][PY][PY] = tc_int[PY][PY][PX][PX] =
        m_int[q_q] + m_int[q_Qpp] + m_int[Qpp_q] + m_int[Qxx_Qyy];

    tc_int[PX][PX][PZ][PZ] = tc_int[PY][PY][PZ][PZ] =
        m_int[q_q] + m_int[q_Qzz] + m_int[Qpp_q] + m_int[Qpp_Qzz];

    tc_int[PZ][PZ][PX][PX] = tc_int[PZ][PZ][PY][PY] =
        m_int[q_q] + m_int[q_Qpp] + m_int[Qzz_q] + m_int[Qzz_Qpp];

    tc_int[PZ][PZ][PZ][PZ] =
        m_int[q_q] + m_int[q_Qzz] + m_int[Qzz_q] + m_int[Qzz_Qzz];

    tc_int[PX][PZ][PX][PZ] = tc_int[PY][PZ][PY][PZ] =
    tc_int[PZ][PX][PX][PZ] = tc_int[PZ][PY][PY][PZ] =
    tc_int[PX][PZ][PZ][PX] = tc_int[PY][PZ][PZ][PY] =
    tc_int[PZ][PX][PZ][PX] = tc_int[PZ][PY][PZ][PY] = m_int[Qpz_Qpz];

    tc_int[PX][PY][PX][PY] = tc_int[PY][PX][PX][PY] =
    tc_int[PX][PY][PY][PX] = tc_int[PY][PX][PY][PX] = m_int[Qxy_Qxy];
}
