#!/usr/local/bin/perl -w
#
# $Id: energies.pl,v 1.6 2000/11/09 19:44:53 jens Exp $
#
# Copyright (C) 2001 Jens Thoms Toerring
#
# g_rhf is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# g_rhf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with g_rhf; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Parses a RHF MOPAC or AMPAC or Hyperchem/Plato ZINDO/S result file and
# prints all orbital energies (MOPAC, AMPAC) or state energies (ZINDO/S)


use strict;

local *F;
my $method;


open( F, "< $ARGV[ 0 ]" ) or exit;

while ( <F> ) {

	if ( /\b((MOPAC)|(AMPAC)|(HyperChem))\b/ ) {
		$method = uc $1;
		last;
	}
}

exit unless defined $method;

if ( $method =~ /(AMPAC)|(MOPAC)/ ) {

	my @list;

	while ( <F> ) {
		if ( /\s*Root/i ) {
			my $i = 0;
			while ( <F> ) {
				$i++ unless /^\s*$/;
				last if $i == 2;
			}
			/\s*(.*)\b\s*/;             # get rid of leading or trailing spaces
			@list = split /\s+/, $1;
			print "$_\n" foreach ( @list );
		}
	}
} else {

	my @somo_to_unocc;
	my @occ_to_somo;

	while ( <F> ) {

		if ( /Type 1 \(M->A\):/ ) {
			while ( <F> ) {
				next if /^\s*$/;           # skip black lines
				last unless /^\s*\d/;      # stop on next tilte line
				/\s*(.*)\b\s*/;
				push @somo_to_unocc, split /\s+/, $1;
			}
			while ( <F> ) {
				next if /^\s*$/;
				last unless /^\s*\d/;
				/\s*(.*)\b\s*/;
				push @occ_to_somo, split /\s+/, $1;
			}
			last;
		}
	}

	print -$_, "\n" foreach reverse @occ_to_somo;
	print "0.0\n";
	print "$_\n" foreach @somo_to_unocc;
}

close F;
