#!/usr/local/bin/perl -w
#
# $Id: lcaos.pl,v 1.3 2000/11/09 19:44:54 jens Exp $
#
# Copyright (C) 2001 Jens Thoms Toerring
#
# g_rhf is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# g_rhf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with g_rhf; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Parses a MOPAC or AMPAC RHF or Hyperchem/Plato ZINDO/S result file and
# prints the orbital number of the last LCAO coefficfient in the actual line,
# the number of LCAO coefficients in this line, the type of the orbital
# (S = 0, Px = 1, Py = 2, Pz = 3) and the number of the atom, followed by
# all LCAO coefficients in this line.

use strict;

local *F;
my %orbital_types = ( S  => 0,
					  PX => 1,
					  PY => 2,
					  PZ => 3 );
my $current_orbital_type;
my @list;
my $last_state = 0;
my $need_to_add = 0;


open( F, "< $ARGV[ 0 ]" ) or exit;

while ( <F> ) {

	# All lines with LCOA coefficients start with the type of the orbital
	# (s, px, py, pz), followed by the atom type (e.g. H, O or Cl) and
	# the atoms number

	if ( /^\s*((S)|(PX)|(PY)|(PZ))\s+[A-Z]{1,2}\s+[0-9]+/i ) {

		$current_orbital_type = $orbital_types{ uc $1 };

		/\s*([SP].*\d)\b\s*/;           # get rid of leading or trailing spaces
		@list = split /\s+/, $1;

		if ( $need_to_add ) {          # new set of LCAO coefficients?
			$last_state += @list - 3;
			$need_to_add = 0;
		}

		print "$last_state ", @list - 3, " $current_orbital_type $list[ 2 ]\n";
		print "$list[ $_ ]\n" foreach ( 3 .. @list - 1 );

	} elsif ( ! /^\s*$/ ) {            # start of new set of LCAO coefficients?
		$need_to_add = 1;
	}
}

close F;
